#include <stdio.h>
#include <avs/flow.h>
#include <avs/avs_data.h>
#include <avs/field.h>
#include <avs/geom.h>

AVSinit_modules()
{
	int polygon_to_geom();

	AVSmodule_from_desc(polygon_to_geom);
}

polygon_to_geom()
{
	int polygon_compute();	/* declare the compute function (below) */
	int out_port;	/* temporaries to hold the port numbers */
	int parm, rarm;

	AVSset_module_name("Plot Poly", MODULE_DATA);

	out_port = AVScreate_output_port(/*name*/ "Geometry", /*type*/"geom");

	parm = AVSadd_parameter("Poly filename", "string", NULL, NULL, NULL);
   	AVSconnect_widget(parm,"browser");

	rarm = AVSadd_float_parameter("Node Size", 0.01, 0.0, 1000.0);
	AVSconnect_widget(rarm,"typein_real");

	/* Tell avs what subroutine to call to do the compute */
	AVSset_compute_proc(polygon_compute);
}

float rgb[8][3] =  {
	 0.990, 0.000, 0.000,
	 0.990, 0.559, 0.000,
	 0.542, 0.990, 0.000,
	 0.000, 0.990, 0.630,
	 0.000, 0.364, 0.990,
	 0.755, 0.990, 0.000,
	 0.990, 0.001, 0.000,
	 0.990, 0.990, 0.990};

#define MAXVERTS		10
#define MAXCOLOR		8

polygon_compute(output, filename, ptrradius)
GEOMedit_list *output;
char *filename;
float *ptrradius;
{
	GEOMobj *faces, *edges, *nodes;
	FILE *fp;
	int i, nvert, ipoint, icolor, npoint = 0;
	float *radii, *r, *c;
	float verts[MAXVERTS][3], col[MAXVERTS][3];
	char name[16];

	if (filename == NULL) return(0); 
	if ((fp = fopen(filename,"r")) == NULL) return(0); 

	faces = GEOMcreate_obj(GEOM_POLYHEDRON,NULL);
	edges = GEOMcreate_obj(GEOM_POLYTRI,NULL);

	while(!feof(fp)){
		fscanf(fp, "%d %d", &icolor, &nvert);
		if(icolor < 0){
printf("gouraud nvert=%d\n", nvert);
			for(i=0; i<nvert; i++)
				fscanf(fp, "%f %f %f %f %f %f", 
				&verts[i][0], &verts[i][1], &verts[i][2],
				&col[i][0], &col[i][1], &col[i][2]);
printf("data read\n");
		} else {
			for(i=0; i<nvert; i++){
				fscanf(fp, "%f %f %f", 
				&verts[i][0], &verts[i][1], &verts[i][2]);
				memcpy(&col[i][0], &rgb[icolor][0], 3*sizeof(float));
			}
		}

		switch(nvert){
			case 1:
				npoint++;
				break;
			case 2:
				GEOMadd_disjoint_line(edges,verts,col,2,0);
				break;
			default:
				GEOMadd_disjoint_polygon(faces,verts,NULL,col,nvert,GEOM_NOT_SHARED,0);
		}
	}

	if(npoint){
		radii = (float *)malloc(npoint*sizeof(float));
		r = (float *)malloc(3*npoint*sizeof(float));
		c = (float *)malloc(3*npoint*sizeof(float));
		rewind(fp);
		ipoint = 0;
		while(!feof(fp)){
			fscanf(fp, "%d %d", &icolor, &nvert);
			if(icolor<0){
			for(i=0; i<nvert; i++)
				fscanf(fp, "%f %f %f %f %f %f", 
					&verts[i][0], &verts[i][1], &verts[i][2],
					&col[i][0], &col[i][1], &col[i][2]);
			} else {
			for(i=0; i<nvert; i++)
				fscanf(fp, "%f %f %f", 
					&verts[i][0], &verts[i][1], &verts[i][2]);
			}
			if(nvert != 1)
				continue;
			memcpy(&r[3*ipoint], &verts[0][0], 3*sizeof(float));
			memcpy(&c[3*ipoint], &rgb[icolor][0], 3*sizeof(float));
			radii[ipoint++] = *ptrradius;
		}
		nodes = GEOMcreate_sphere(NULL, r, radii, NULL, NULL, npoint, 0);
		GEOMadd_float_colors(nodes, c, npoint, 0);
	}

	GEOMgen_normals(faces, GEOM_FACET_NORMALS);

	*output = GEOMinit_edit_list(*output);
	GEOMedit_geometry(*output,"faces",faces);
	GEOMedit_geometry(*output,"edges",edges);
	if(npoint)
		GEOMedit_geometry(*output,"nodes",nodes);

	GEOMdestroy_obj(faces); 
	GEOMdestroy_obj(edges);
	GEOMdestroy_obj(nodes);

	return(1);
}
