/*    These are the user specific parts of mesh input/output */

#include "strings.h"
#include <string.h>

#define RANDOM  0
#define EDGE    1



read_string()
{
	char string[100];
	double lambda;
	int i=0, islash, ibar=0,mode;
	FILE *in;

    printf("Input Filename (without suffix)?\n");
    get_string(string);

#ifdef CFS
   strcat(string, ".ct");
   sprintf(string, "/cfs/paulc/strings/%s", string);
#else
    /* Don't actually need this, except maybe as a check */
    if ((strncmp(string,"icos",4) != 0) && (strncmp(string,"torus",5) != 0)) {
	islash = -1;
	while(string[i] != '\0'){
		if(string[i] == '/')
			islash = i;
		i++;
	}
	while(string[ibar++] != '_');
	sscanf(string+ibar, "%d", &dim);
	sscanf(string+islash+6, "%lf", &lambda);
	if(string[islash+5] == 'A'){
		lambdaA = lambda;
		lambdaE = 0.0;
	} else {
		lambdaA = 0.0;
		lambdaE = lambda;
	}
	printf("Dim = %d, lambda%c = %f\n", dim, string[islash+5], lambda);
        fflush(stdout);
    }
    strcat(string, ".ct");
#endif CFS

    if((in = fopen(string, "r")) == (FILE *)NULL){
        printf("Cannot open %s\n", string);
		exit(0);
    }
    else {
        readct(in);
    }

	fill_user_structs();
}

write_string()
{
	char filename[100];
	FILE *write;

	if(lambdaE == 0.0)
		sprintf(filename, "N%d/MeshA%4.3f_%d.ct", nnode, lambdaA, dim);
	else
		sprintf(filename, "N%d/MeshE%4.3f_%d.ct", nnode, lambdaE, dim);

#ifdef CFS
   sprintf(filename, "/cfs/paulc/strings/%s", filename);
#endif CFS

#ifdef DEBUG
	printf("Writing mesh to %s\n", filename);
#endif DEBUG
	if((write = fopen(filename, "w")) == NULL){
		perror(filename);
		return;
	} else
		writect(write);

#ifdef GFX
	if(lambdaE == 0.0)
		sprintf(filename, "N%d/MeshA%4.3f_%d.gfx", nnode, lambdaA, dim);
	else
		sprintf(filename, "N%d/MeshE%4.3f_%d.gfx", nnode, lambdaE, dim);
	if((write = fopen(filename, "w")) == NULL){
		perror(filename);
		return;
	} else
		writegfx(write);
#endif GFX
}



string_read(fp, type, ptr)
FILE *fp;
int type;
char *ptr;
{
    NODEPTR n;
    ELMTPTR e;
    BDYPTR b;
	int i;

    switch(type){
		case MISC:
			return;
        case ELMT:
            e = (ELMTPTR)ptr;
            return;
        case NODE:
            n = (NODEPTR)ptr;
#ifdef DEBUG
printf("(%d)", procnum);
#endif
			for(i=0; i<dim; i++){
				fscanf(fp, "%lf", &n->user->r[i]);
#ifdef DEBUG
printf(" %f ", n->user->r[i]);
#endif
			}
#ifdef DEBUG
printf("\n");
fflush(stdout);
#endif
            return;
        case BDY:
            b = (BDYPTR)ptr;
            return;
    }
}


string_write(fp, type, ptr)
FILE *fp;
int type;
char *ptr;
{
    NODEPTR n;
    ELMTPTR e;
    BDYPTR b;
    int i;
    float color[3];

    switch(type){
		case MISC:
			fprintf(fp, "\n");
			return;
        case ELMT:
            e = (ELMTPTR)ptr;
			fprintf(fp, "\n");
            return;
        case NODE:
            n = (NODEPTR)ptr;
			for(i=0; i<dim; i++)
				fprintf(fp, "%f ", n->user->r[i]);
			fprintf(fp, "\n");
            return;
        case GFX:
            n = (NODEPTR)ptr;
	    /* print out positions for this node */
		for(i=0; i<dim; i++)
			fprintf(fp, "%f ", n->user->r[i]);
		fprintf(fp, "  ");
	    /* print out rgb values (floats from 0 to 1) for this node */
		set_color(RANDOM,color);
		/*
		set_color(EDGE,color);
		*/
		fprintf(fp, "%f ", color[0]);
		fprintf(fp, "%f ", color[1]);
		fprintf(fp, "%f ", color[2]);
		fprintf(fp, "\n");
            return;
        case BDY:
            b = (BDYPTR)ptr;
			fprintf(fp, "\n");
            return;
    }
}



set_color(method,color)
int method;
float color[3];
{
   double pranf();

   if (method == RANDOM) {
      color[0] = pranf();
      color[1] = pranf();
      color[2] = pranf();
   }
   else if (method == EDGE) {
      /* Try to figure out how to map edge action values to colors */
   }

}


/* Write out the mesh in a form suitable for reading in to the
   AVS graphics program. Uses the standard user_write form of
   DIME, with an extra case (GFX) added to the user_write 
   function (which in our case is string_write).
 */

writegfx(out)
FILE *out;
{
	NODEPTR node;
	ELMTPTR elmt;
	ALIASPTR ali, astart;
	int i, ibdy, dest, num;

#ifdef CUBIX
        fmulti(out);
#endif CUBIX

	FORALLELMTS(elmt)
	   	fprintf(out, "-1 3  ");
	   	for(i=0; i<3; i++) {
		   (*user_write)(out, GFX, (char *)elmt->neigh[i]);
		}
   	   	fprintf(out, "\n");
	NEXTELMT(elmt)
	fflush(out);
	fclose(out);

}

